
#include <Y/y.h>
#include <Y/util/yutil.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <Y/main/config.h>

#include <Y/modules/module.h>
#include <Y/input/keymap.h>

static void
configHandle (char *action, int argc, char **argv)
{
  if (strcmp (action, "LoadModule") == 0)
    {
      if (argc > 0)
        moduleLoad (argv[0], argc-1, (const char **)(argv+1));
    }
  else if (strcmp (action, "LoadKeymap") == 0)
    {
      if (argc > 0)
        keymapLoad (argv[0]);
    }
}

static void
configParseLine (char *line)
{
  int   argc;
  char *argv[128];

  size_t l;
  char *lp;

  if (line[0] == '#')
    return;

  argc = 0;
  l = strcspn (line, " \t");
  if (l == 0)
    return;
  lp = line + l;
  if (*lp != '\0')
    {
      *(lp++) = '\0';
      while (argc < 128)
        { 
          lp += strspn (lp, " \t");

          if (*lp == '\0')
            break;
          else if (*lp == '"')
            {
              argv[argc++] = ++lp;
              lp = strchr (lp, '"');
              if (lp == NULL)
                {
                  Y_WARN ("Mismatched Quote");
                  break;
                }
              *lp++ = '\0';
            }
          else
            {
              argv[argc++] = lp;
              lp += strcspn (lp, " \t");
              if (*lp == '\0')
                break;
              *lp++ = '\0';
            }
        }
    }
  configHandle (line, argc, argv);
}

void
configLoad (const char *configFilename)
{
  char buffer[1024];
  FILE *file;
  int filed;
  size_t ret;
  int position;

  Y_TRACE ("Opening config file: %s", configFilename);

  
  filed = open (configFilename, O_RDONLY | O_NOCTTY);
  if (filed < 0)
    {
      Y_ERROR ("Error opening configuration file %s: %s", configFilename,
               strerror (errno));
      return;
    }

  file = fdopen (filed, "r");

  position = 0;
  do
    {
      ret = fread (buffer + position, sizeof (char), 1, file);
      if (ret > 0)
        {
          if (buffer[position] == '\n')
            {
              if (position == 0 || buffer[position - 1] != '\\')
                {
                  buffer[position] = '\0';
                  configParseLine (buffer);
                  position = 0;
                }
              else
                --position;
            }
          else
            {
              ++position;
              if (position >= 1023)
                position = 1023;
            }
        }
    }
  while (ret > 0);

  if (ferror (file))
    Y_ERROR ("Error reading configuration file %s: %s", configFilename, strerror (errno));

  fclose (file);
}

void
configLoadDefault ()
{
  char *configFile = getenv ("YCONFIGFILE");
  if (configFile == NULL)
    {
      int configDirLen = strlen (yConfigDir);
      char configDefault[configDirLen + 16];
      strcpy (configDefault, yConfigDir);
      strcat (configDefault, "/default.conf");
      configLoad (configDefault);
    }
  else
    {
      configLoad (configFile);
    }
}

/* arch-tag: 2145afe4-a40d-452c-a9a9-7f53d5ed8475
 */
