#include <Y/y.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <Y/screen/screen.h>
#include <Y/main/control.h>
#include <Y/main/config.h>
#include <Y/input/keymap.h>
#include <Y/object/class.h>
#include <Y/object/object.h>
#include <Y/object/parameters.h>
#include <Y/message/special.h>
#include <Y/text/font.h>

#include <Y/widget/window.h>
#include <Y/widget/menu.h>

static void
final ()
{
  moduleFinalise ();
  keymapFinalise ();
  classFinalise ();
  fontFinalise ();
  screenFinalise ();
  controlFinalise ();
  specialFinalise ();
}

int
main (int argc, char **argv)
{
  const char *configFile = NULL;
  int i;
  int detaching = 1;

  pid_t pid;
  int pipedes[2];
  int status[2];

  Y_LOG_INIT ();

  for (i=1; i<argc; ++i)
    {
      if (strncmp (argv[i], "--config=", 9) == 0)
        configFile = argv[i] + 9;
      if (strcmp (argv[i], "--no-detach") == 0)
        detaching = 0;
    }

  if (detaching)
    {
      if (pipe (pipedes) < 0)
        {
          fprintf (stderr, "Could not open pipe: %s", strerror (errno));
          return EXIT_FAILURE;
        }
    
      /* Detach from the console */
      pid = fork ();
      if (pid == -1)
        {
          Y_FATAL ("Fork failed: %s", strerror (errno));
          return EXIT_FAILURE;
        }
      else if (pid != 0)
        {
          /* wait for child to finish initialising... */
          int returns[2];
          if (read (pipedes[0], returns, sizeof (int) * 2) > 0)
            {
              if (returns[1] > 0)
                {
                  char buffer[returns[1]+1];
                  read (pipedes[0], buffer, returns[1]);
                  buffer[returns[1]] = '\0';
                  fprintf (stderr, "Y: %s\n", buffer);
                }
              return (returns[0]);
            }
          else
            {
              /* lost contact with the child? */
              perror ("Y: Error whilst detaching:");
              return EXIT_FAILURE;  
            }
        }
    
      fclose (stdin);
      fclose (stdout);
      fclose (stderr);
      setsid ();

    }

  specialInitialise ();
  controlInitialise ();
  screenInitialise ();
  fontInitialise ();
  classInitialise ();
  keymapInitialise ();
  moduleInitialise ();

  atexit (final);

  srandom (time (NULL) ^ getpid ());

  if (configFile != NULL && strlen (configFile) > 0)
    configLoad (configFile);
  else
    configLoadDefault ();

  if (detaching)
    {
      /* signal to the parent process that all is well */
      status[0] = EXIT_SUCCESS;
      status[1] = 0;
      write (pipedes[1], status, sizeof (int) * 2);

      close (pipedes[0]);
      close (pipedes[1]);

      /* we are now fully detached... */
    }

  controlRun ();

  return EXIT_SUCCESS;
}


/* arch-tag: cfcca386-0d95-4369-8d50-a2c785bf1231
 */
