
#include <Y/y.h>
#include <Y/util/yutil.h>
#include <Y/message/special.h>
#include <Y/util/index.h>
#include <string.h>

struct Special
{
  char *name;
  void (*callback)(struct Parameters *);
};

static struct Index *specialIndex = NULL;

static int
specialKeyFunction (void *key_v, void *obj_v)
{
  const char *key = key_v;
  struct Special *obj = obj_v;
  return strcmp (key, obj->name);
}

static int
specialComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Special *obj1 = obj1_v;
  struct Special *obj2 = obj2_v;
  return strcmp (obj1->name, obj2->name);
}

static void
specialDestructorFunction (void *obj_v)
{
  struct Special *obj = obj_v;
  yfree (obj -> name);
  yfree (obj);
}

void
specialInitialise ()
{
  specialIndex = indexCreate (specialKeyFunction, specialComparisonFunction);
}

void
specialFinalise ()
{
  indexDestroy (specialIndex, specialDestructorFunction);
}

void
specialRegister (const char *name, void (*callback)(struct Parameters *))
{
  struct Special *special = ymalloc (sizeof (struct Special));
  special -> name = ystrdup (name);
  special -> callback = callback;
  indexAdd (specialIndex, special);
}

void
specialUnregister (const char *name)
{
  struct Special *special = indexRemove (specialIndex, (void *)name);
  if (special != NULL)
    {
      yfree (special -> name);
      yfree (special);
    }
}

void
(*specialFind (const char *name))(struct Parameters *)
{
  struct Special *special = indexFind (specialIndex, (void *)name);
  if (special != NULL)
    return special -> callback;
  else
    return NULL;
}

/* arch-tag: 6d654a42-c605-4719-8d1a-38c2777d0dfa
 */
