#include <Y/modules/theme.h>
#include <Y/modules/theme_interface.h>

#include <string.h>

static struct Theme *topTheme = NULL;

void
themeAdd (struct Theme *theme)
{
  theme -> nextTheme = topTheme;
  topTheme = theme;
}

void
themeRemove (struct Theme *theme)
{
  struct Theme * current = topTheme;

  if (topTheme == NULL)
    return;

  if (topTheme == theme)
    {
      topTheme = topTheme -> nextTheme;
      return;
    }

  while (current -> nextTheme != NULL)
    {
      if (current -> nextTheme == theme)
        {
          current -> nextTheme = current -> nextTheme -> nextTheme;
          return;
        }
      current = current -> nextTheme;
    }
}


void
themeDrawBackgroundPane (struct Painter *p, int x, int y, int w, int h)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> drawBackgroundPane != NULL)
        return current -> drawBackgroundPane (p, x, y, w, h);
      current = current -> nextTheme;
    }
}

void
themeDrawButtonPane (struct Painter *p, int x, int y, int w, int h,
                     enum WidgetState state)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> drawButtonPane != NULL)
        return current -> drawButtonPane (p, x, y, w, h, state);
      current = current -> nextTheme;
    }
}

struct Font *
themeGetDefaultFont ()
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> getDefaultFont != NULL)
        return current -> getDefaultFont ();
      current = current -> nextTheme;
    }
  return NULL;
}


void 
themeWindowInit (struct Window *w)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowInit != NULL)
        return current -> windowInit (w);
      current = current -> nextTheme;
    }
}

void
themeWindowPaint (struct Window *w, struct Painter *p)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowPaint != NULL)
        return current -> windowPaint (w, p);
      current = current -> nextTheme;
    }
}

int
themeWindowGetRegion (struct Window *w, int x, int y)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowGetRegion != NULL)
        return current -> windowGetRegion (w, x, y);
      current = current -> nextTheme;
    }
  return 0;
}

int
themeWindowPointerMotion (struct Window *w, int x, int y, int dx, int dy)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowPointerMotion != NULL)
        return current -> windowPointerMotion (w, x, y, dx, dy);
      current = current -> nextTheme;
    }
  return 0;
}

int
themeWindowPointerButton (struct Window *w, int x, int y, int b, int s)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowPointerButton != NULL)
        return current -> windowPointerButton (w, x, y, b, s);
      current = current -> nextTheme;
    }
  return 0;
}

void
themeWindowReconfigure (struct Window *w,
                        int *minWidth_p, int *minHeight_p,
                        int *reqWidth_p, int *reqHeight_p,
                        int *maxWidth_p, int *maxHeight_p)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowReconfigure != NULL)
        return current -> windowReconfigure (w, minWidth_p, minHeight_p,
                                                reqWidth_p, reqHeight_p,
                                                maxWidth_p, maxHeight_p);
      current = current -> nextTheme;
    }
}

void themeWindowResize (struct Window *w)
{
  struct Theme *current = topTheme;

  while (current != NULL)
    {
      if (current -> windowResize != NULL)
        return current -> windowResize (w);
      current = current -> nextTheme;
    }
}


/* arch-tag: 95fe2cbe-4eb4-40f1-91ac-077f990e1e5a
 */
