
#include <Y/y.h>
#include <Y/object/object_p.h>
#include <Y/const.h>
#include <Y/object/class_p.h>
#include <Y/util/yutil.h>
#include <stdlib.h>
#include <string.h>

static int objectNextID = 1;
static struct Index *objectIndex = NULL;

int
objectKeyFunction (void *key_v, void *obj_v)
{
  int *key = key_v;
  struct Object *obj = obj_v;
  if (*key == obj -> oid)
    return 0;
  else if (*key < obj -> oid)
    return -1;
  else
    return 1; 
}

int
objectComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Object *obj1 = obj1_v;
  struct Object *obj2 = obj2_v;
  if (obj1 -> oid == obj2 -> oid)
    return 0;
  else if (obj1 -> oid < obj2 -> oid)
    return -1;
  else
    return 1; 
}

void
objectInitialise (struct Object *o, struct Class *c)
{
  o -> oid = objectNextID++;
  o -> c = c;
  o -> client = NULL;
  o -> properties = indexCreate (propertyKeyFunction,
                                 propertyComparisonFunction);
  o -> signals = indexCreate ((int (*)(void *, void *))strcmp,
                              (int (*)(void *, void *))strcmp);
  if (objectIndex == NULL)
    objectIndex = indexCreate (objectKeyFunction, objectComparisonFunction);
  indexAdd (objectIndex, o);
}

void
objectFinalise (struct Object *o)
{
  indexRemove (objectIndex, &(o -> oid)); 
  indexDestroy (o -> properties, propertyDestructorFunction); 
  indexDestroy (o -> signals, yfree);
  if (o->client != NULL)
    clientRemoveObject (o -> client, o);
  if (indexCount (objectIndex) == 0)
    {
      indexDestroy (objectIndex, NULL);
      objectIndex = NULL;
    }
}

void
objectInvokeMethod (struct Object *o, const char *method,
                    struct Parameters *p)
{
  classInvokeMethod (o -> c, o, method, p);
}

int
objectGetID (struct Object *o)
{
  return o -> oid;
}

struct Class *
objectClass (struct Object *o)
{
  return o -> c;
}

struct Object *
objectFind (int oid)
{
  struct Object *obj = indexFind (objectIndex, &oid);
  return obj;
}

void
objectEmitSignal (struct Object *o, const char *name, struct Parameters *params)
{
  if (indexFind (o->signals, (void *)name) != NULL)
    {
      struct Message *m;
      int len;
      char *data;
      if (params != NULL)
        data = parametersGetOutputData (params, name, &len);
      else
        {
          data = ystrdup (name);
          len = strlen (name) + 1;
        }
      m = messageCreate (clientGetID (o -> client), 0, o->c->id, o->oid,
                         YMO_EVENT, 0, 0, len, data);
      messageDespatch (NULL, m);
    }
}

void
objectSubscribeSignal (struct Object *o, const char *name)
{
  if (indexFind (o->signals, (void *)name) == NULL)
    indexAdd (o->signals, ystrdup (name));
}

const char *
objectGetPropertyAsString (struct Object *o, const char *name)
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    return NULL;
  return propertyGetAsString (prop);
}

int
objectGetPropertyAsInteger (struct Object *o, const char *name)
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    return 0;
  return propertyGetAsInteger (prop);
}

int
objectIsPropertySet (struct Object *o, const char *name)
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    return 0;
  else
    return 1;
}

void
objectClearProperty (struct Object *o, const char *name)
{
  struct Property *prop = indexRemove (o -> properties, (void *)name);
  if (prop != NULL)
    propertyDestroy (prop);
}

void
objectSetPropertyToString (struct Object *o, const char *name,
                           const char *value)
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    {
      prop = propertyCreate (name);
      indexAdd (o -> properties, prop);
    }
  propertySetToString (prop, value);
}

void
objectSetPropertyToInteger (struct Object *o, const char *name, int value)
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    {
      prop = propertyCreate (name);
      indexAdd (o -> properties, prop);
    }
  propertySetToInteger (prop, value);
}

void
objectPropertyCallOnSet (struct Object *o, const char *name, void *data,
                         void (*callback)(struct Property *, void *))
{
  struct Property *prop = indexFind (o -> properties, (void *)name);
  if (prop == NULL)
    {
      prop = propertyCreate (name);
      indexAdd (o -> properties, prop);
    }
  propertyCallOnSet (prop, data, callback);
}

void
objectDestroy (struct Object *o)
{
  if (o != NULL && o -> c != NULL && o -> c -> destroy != NULL)
    o -> c -> destroy (o);
}

/* arch-tag: c4962764-6b04-48f7-95bc-91d371fe3bb7
 */
