#include <Y/y.h>
#include <Y/object/property.h>
#include <Y/util/yutil.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

struct Property
{
  char *name;
  int flags;
  char *valueString;
  int   valueInteger;
  void (*onSet)(struct Property *, void *data);
  void (*onGet)(struct Property *, void *data);
  void  *onSetData;
  void  *onGetData;
};

#define  Y_PROPERTY_FLAGS_VALID_STRING   (1<<0)
#define  Y_PROPERTY_FLAGS_VALID_INTEGER  (1<<1)

struct Property *
propertyCreate (const char *name)
{
  struct Property *prop = ymalloc (sizeof (struct Property));
  prop -> name = ymalloc (strlen (name) + 1);
  strcpy (prop -> name, name);
  prop -> flags = 0;
  prop -> valueString = NULL;
  prop -> valueInteger = 0;
  prop -> onSet = NULL;
  prop -> onGet = NULL;
  prop -> onSetData = NULL;
  prop -> onGetData = NULL;
  return prop;
}

void
propertyDestroy (struct Property *self)
{
  yfree (self -> name);
  yfree (self -> valueString);
  yfree (self);
}

int
propertyKeyFunction (void *key_v, void *obj_v)
{
  const char *key = key_v;
  struct Property *obj = obj_v;
  return strcmp (key, obj -> name);
}

int
propertyComparisonFunction (void *obj1_v, void *obj2_v)
{
  struct Property *obj1 = obj1_v;
  struct Property *obj2 = obj2_v;
  return strcmp (obj1 -> name, obj2 -> name);
}

void
propertyDestructorFunction (void *obj)
{
  struct Property *prop = obj;
  propertyDestroy (prop);
}

const char *
propertyGetAsString (struct Property *self)
{
  if (self -> onGet != NULL)
    self -> onGet (self, self -> onGetData);

  if (self -> flags & Y_PROPERTY_FLAGS_VALID_STRING)
    return self -> valueString;

  if (self -> flags & Y_PROPERTY_FLAGS_VALID_INTEGER)
    {
      yfree (self -> valueString);
      self -> valueString = ymalloc (64);
      snprintf (self -> valueString, 64, "%d", self -> valueInteger);
      self -> valueString[63] = '\0';
      self -> flags |= Y_PROPERTY_FLAGS_VALID_STRING;
      return self -> valueString;
    }

  return NULL;
}

int
propertyGetAsInteger (struct Property *self)
{
  if (self -> onGet != NULL)
    self -> onGet (self, self -> onGetData);

  if (self -> flags & Y_PROPERTY_FLAGS_VALID_INTEGER)
    return self -> valueInteger;

  if (self -> flags & Y_PROPERTY_FLAGS_VALID_STRING)
    {
      self -> valueInteger = strtol (self -> valueString, NULL, 0);
      if (self -> valueInteger == 0)
        {
          if (strcasecmp (self -> valueString, "yes") == 0)
            self -> valueInteger = 1;
        }
      self -> flags |= Y_PROPERTY_FLAGS_VALID_INTEGER;
      return self -> valueInteger;
    }

  return 0;
}

void
propertySetToString (struct Property *self, const char *value)
{
  yfree (self -> valueString);
  self -> valueString = ymalloc (strlen (value) + 1);
  strcpy (self -> valueString, value);
  self -> flags &= ~Y_PROPERTY_FLAGS_VALID_INTEGER;
  self -> flags |= Y_PROPERTY_FLAGS_VALID_STRING;
  if (self -> onSet != NULL)
    self -> onSet (self, self -> onSetData);
}

void
propertySetToInteger (struct Property *self, int value)
{
  yfree (self -> valueString);
  self -> valueInteger = value;
  self -> flags &= ~Y_PROPERTY_FLAGS_VALID_STRING;
  self -> flags |= Y_PROPERTY_FLAGS_VALID_INTEGER;
  if (self -> onSet != NULL)
     self -> onSet (self, self -> onSetData);
}

void
propertyCallOnSet (struct Property *self, void *data,
                   void (*callback)(struct Property *, void *))
{
  self -> onSet = callback;
  self -> onSetData = data;
} 

void
propertyCallOnGet (struct Property *self, void *data,
                   void (*callback)(struct Property *, void *))
{
  self -> onGet = callback;
  self -> onGetData = data;
} 

/* arch-tag: d6e26a45-3a53-4a66-984b-73d4e033d7fc
 */
