#include <Y/widget/checkbox.h>
#include <Y/widget/widget_p.h>

#include <Y/util/yutil.h>
#include <Y/buffer/painter.h>

#include <Y/modules/theme.h>

#include <Y/object/class_p.h>
#include <Y/object/object_p.h>

#include <Y/text/font.h>

#include <stdio.h>
#include <ctype.h>
#include <string.h>

struct CheckBox
{
  struct Widget widget;
};

static void checkboxReconfigure (struct Widget *);
static void checkboxPaint (struct Widget *, struct Painter *);
static int checkboxPointerButton (struct Widget *, int, int, int, int);
static int checkboxPointerMotion (struct Widget *, int, int, int, int);
static void checkboxPointerEnter (struct Widget *, int, int);
static void checkboxPointerLeave (struct Widget *);

static struct Object *checkboxInstantiate (struct Parameters *);

static void checkboxTextSet (struct Property *, void *);
static void checkboxCheckedSet (struct Property *, void *);

CLASSDEF struct Class checkboxClass =
{
  name:          "CheckBox",
  super:         &widgetClass,
  instantiate:   checkboxInstantiate,
  destroy:       (void(*)(struct Object *))checkboxDestroy,
  methodCount:   0,
  methods:
    { {} }
};

static struct WidgetTable checkboxTable =
{
  c:             &checkboxClass,
  reconfigure:   checkboxReconfigure,
  paint:         checkboxPaint,
  pointerButton: checkboxPointerButton,
  pointerMotion: checkboxPointerMotion,
  pointerEnter:  checkboxPointerEnter,
  pointerLeave:  checkboxPointerLeave
};

static inline struct CheckBox *
castBack (struct Widget *widget)
{
  /* assert ( widget -> c == checkboxClass ); */
  return (struct CheckBox *)widget;
}

static inline const struct CheckBox *
castBackConst (const struct Widget *widget)
{
  /* assert ( widget -> c == checkboxClass ); */
  return (const struct CheckBox *)widget;
}

static void
checkboxPaint (struct Widget *self_w, struct Painter *painter)
{
  struct Font *font;
  struct CheckBox *self = castBack (self_w);
  int offset, ascender, descender;
  const char *text =
               objectGetPropertyAsString (widgetToObject (self_w), "text");

  painterSaveState (painter);
  painterSetPenColour (painter, 0xFF000000);
  font = themeGetDefaultFont ();
  fontGetMetrics (font, &ascender, &descender, NULL);
  fontMeasureString (font, text, &offset, NULL, NULL);
  fontRenderString (font, painter, text, 20 - offset,
                    (self_w -> h - ascender - descender) / 2 + ascender);
  themeDrawButtonPane (painter, 0, (self -> widget.h - 16) / 2,
                                16, 16, self -> widget.state);
  if (objectGetPropertyAsInteger (widgetToObject (self_w), "checked"))
    {
      int yp = (self -> widget.h - 16) / 2;
      painterDrawLine (painter, 3, yp + 7, 3, 6); 
      painterDrawLine (painter, 6, yp + 13, 6, -12); 
    }
  painterRestoreState (painter);
}

struct CheckBox *
checkboxCreate ()
{
  struct CheckBox *self = ymalloc (sizeof (struct CheckBox));
  objectInitialise (&(self -> widget.o), &checkboxClass);
  widgetInitialise (&(self -> widget), &checkboxTable);
  objectPropertyCallOnSet (checkboxToObject (self), "text", self, checkboxTextSet);
  objectPropertyCallOnSet (checkboxToObject (self), "checked", self, checkboxCheckedSet);
  return self;
}

void
checkboxDestroy (struct CheckBox *self)
{
  widgetFinalise (checkboxToWidget (self));
  objectFinalise (checkboxToObject (self));
  yfree (self);
}

struct Widget *
checkboxToWidget (struct CheckBox *self)
{
  return &(self -> widget);
}

struct Object *
checkboxToObject (struct CheckBox *self)
{
  return &(self -> widget.o);
}

static struct Object *
checkboxInstantiate (struct Parameters *p)
{
  return checkboxToObject (checkboxCreate ());
}

static void checkboxReconfigure (struct Widget *self_w)
{
  struct CheckBox *self = castBack (self_w);
  struct Font *font;
  int offset, width, ascender, descender;
  const char *text =
               objectGetPropertyAsString (widgetToObject (self_w), "text");
  font = themeGetDefaultFont ();
  fontGetMetrics (font, &ascender, &descender, NULL);
  fontMeasureString (font, text, &offset, &width, NULL);

  self -> widget.minWidth = width - offset + 22;
  self -> widget.minHeight = ascender + descender + 2;
  self -> widget.reqWidth = width - offset + 22;
  self -> widget.reqHeight = ascender + descender + 2;

  if (self -> widget.minHeight < 16)
    {
      self -> widget.minHeight = 16;
      self -> widget.reqHeight = 16;
    }

  /* we would prefer to be square rather than narrow... */
  if (self -> widget.reqWidth < self -> widget.reqHeight)
    self -> widget.reqWidth = self -> widget.reqHeight;

  widgetReconfigure (self -> widget.container);
}

static void
checkboxTextSet (struct Property *text, void *self_v)
{
  struct CheckBox *self = self_v;
  widgetReconfigure (checkboxToWidget (self));
  widgetRepaint (checkboxToWidget (self), NULL);
}

static void
checkboxCheckedSet (struct Property *checked, void *self_v)
{
  struct CheckBox *self = self_v;
  widgetRepaint (checkboxToWidget (self), NULL);
}

static int
checkboxPointerButton (struct Widget *self_w, int x, int y, int b, int pressed)
{
  struct CheckBox *self = castBack (self_w);
  if (b==0)
    {
      if (pressed)
        {
          pointerGrab (self_w);
          self -> widget.state = WIDGET_STATE_PRESSED;
          widgetRepaint (self_w, NULL); 
        }
      else
        {
          pointerRelease ();
          if (self -> widget.state == WIDGET_STATE_PRESSED)
            {
              struct Parameters *params = parametersCreate ();
              int value = objectGetPropertyAsInteger (checkboxToObject (self), "checked");
              value = !value;
              objectSetPropertyToInteger (checkboxToObject (self), "checked", value);
              parametersAddOutputInteger (params, value);
              objectEmitSignal (checkboxToObject (self), "clicked", params);
              parametersDestroy (params);
            }
          if (widgetContainsPoint (self_w, x, y))
            self -> widget.state = WIDGET_STATE_HOVER;
          else
            self -> widget.state = WIDGET_STATE_NORMAL;
          widgetRepaint (self_w, NULL);
        }
    }
  return 1;
}

static int
checkboxPointerMotion (struct Widget *self_w, int x, int y, int dx, int dy)
{
  struct CheckBox *self = castBack (self_w);
  if (self -> widget.state == WIDGET_STATE_PRESSED
      && !widgetContainsPoint (self_w, x, y))
    {
      self -> widget.state = WIDGET_STATE_CANCELLING;
      widgetRepaint (self_w, NULL);
    }
  else if (self -> widget.state == WIDGET_STATE_CANCELLING
      && widgetContainsPoint (self_w, x, y))
    {
      self -> widget.state = WIDGET_STATE_PRESSED;
      widgetRepaint (self_w, NULL);
    }
  return 1;
}

static void
checkboxPointerEnter (struct Widget *self_w, int x, int y)
{
  struct CheckBox *self = castBack (self_w);
  self -> widget.state = WIDGET_STATE_HOVER;
  widgetRepaint (checkboxToWidget (self), NULL);
}

static void
checkboxPointerLeave (struct Widget *self_w)
{
  struct CheckBox *self = castBack (self_w);
  self -> widget.state = WIDGET_STATE_NORMAL;
  widgetRepaint (checkboxToWidget (self), NULL);
}


/* arch-tag: 14a3dfdd-2ed1-4460-8630-96cc4a61094d
 */
