dnl
dnl LIBSIGC CHECK script

AC_DEFUN(LIBSIGC_CHECK,
[dnl
dnl Get the cflags and libraries from the libsigc-config program
dnl
AC_ARG_WITH(libsigc-prefix,AC_HELP_STRING([--with-libsigc-prefix],[Prefix where LIBSIGC is installed (optional)]),
            libsigc_prefix="$withval", libsigc_prefix="")
AC_ARG_WITH(libsigc-exec-prefix,AC_HELP_STRING([--with-libsigc-exec-prefix],[Exec prefix where LIBSIGC is installed (optional)]),
            libsigc_exec_prefix="$withval", libsigc_exec_prefix="")
AC_ARG_ENABLE(libsigctest,AC_HELP_STRING([--disable-libsigctest],[Do not try to compile and run a test libsigc program]),
              , enable_libsigctest=yes)

  if test x$libsigc_exec_prefix != x ; then
    libsigc_args="$libsigc_args --exec-prefix=$libsigc_exec_prefix"
    if test x${LIBSIGC_CONFIG+set} != xset ; then
       LIBSIGC_CONFIG=$libsigc_exec_prefix/bin/sigc-config
    fi
  fi
  if test x$libsigc_prefix != x ; then
    libsigc_args="$libsigc_args --prefix=$libsigc_prefix"
    if test x${LIBSIGC_CONFIG+set} != xset ; then
       LIBSIGC_CONFIG=$libsigc_prefix/bin/sigc-config
    fi
  fi

  AC_PATH_PROG(LIBSIGC_CONFIG, sigc-config, no)
  min_libsigc_version=ifelse([$1], ,1.0.0,$1)
  AC_MSG_CHECKING(for libSigC++ - version >= $min_libsigc_version)
  no_libsigc=""
  if test "$LIBSIGC_CONFIG" = "no" ; then
    no_libsigc=yes
  else
    LIBSIGC_CPPFLAGS=`$LIBSIGC_CONFIG $libsigcconf_args --cflags`
    LIBSIGC_LIBS=`$LIBSIGC_CONFIG $libsigcconf_args --libs`
    libsigc_major_version=`$LIBSIGC_CONFIG $libsigcconf_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    libsigc_minor_version=`$LIBSIGC_CONFIG $libsigcconf_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    libsigc_micro_version=`$LIBSIGC_CONFIG $libsigcconf_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    if test "x$enable_libsigctest" = "xyes" ; then
      ac_save_CPPFLAGS="$CPPFLAGS"
      ac_save_LIBS="$LIBS"
      CPPFLAGS="$CPPFLAGS $LIBSIGC_CPPFLAGS"
      LIBS="$LIBS $LIBSIGC_LIBS"

dnl
dnl Now check if the installed LIBSIGC is sufficiently new.
dnl
    rm -f conf.libsigctest
    AC_LANG_PUSH([C++])
    AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <cstring>
#include "sigc++/signal_system.h"

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((std::strlen (str) + 1) * sizeof(char));
      std::strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  { FILE *fp = fopen("conf.libsigctest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_libsigc_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3)
    {
      printf("%s, bad version string\n", "$min_libsigc_version");
      exit(1);
    }

  if (($libsigc_major_version > major) ||
     (($libsigc_major_version == major) && ($libsigc_minor_version > minor)) ||
     (($libsigc_major_version == major) && ($libsigc_minor_version == minor) &&
      ($libsigc_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf ("\n*** 'sigc-config --version' returned %d.%d.%d, but the minimum version\n",
              $libsigc_major_version, $libsigc_minor_version, $libsigc_micro_version);
      printf ("*** of LIBSIGC required is %d.%d.%d.  If sigc-config is correct, then is is\n",
              major, minor, micro);
      printf ("*** best to upgrade to the required version.\n");
      return 1;
    }
}

],, no_libsigc=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
      CPPFLAGS="$ac_save_CPPFLAGS"
      LIBS="$ac_save_LIBS"
    fi
  fi
  if test "x$no_libsigc" = x ; then
    AC_MSG_RESULT(yes)
    AC_SUBST(HAVE_LIBSIGC, 1)
    ifelse([$2], ,:, [$2])
  else
    AC_MSG_RESULT(no)
    if test "$LIBSIGC_CONFIG" = "no" ; then
      echo "*** The sigc-config program installed by libSigC++ could not be found"
      echo "*** If libSigC++ was installed in PREFIX, make sure PREFIX/bin is in"
      echo "*** your path, or set the LIBSIGC_CONFIG environment variable to the"
      echo "*** full path to sigc-config."
    else
      if test -f conf.libsigctest ; then
        :
      else
        echo "*** Could not run LIBSIGC test program, checking why..."
          CPPFLAGS="$CFLAGS $FREETYPE_CPPFLAGS"
          LIBS="$LIBS $FREETYPE_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
#include "sigc++/signal_system.h"
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding libSigC++ or finding the wrong"
          echo "*** version of libSigC++. If it is not finding libSigC++, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
          echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means libSigC++ was incorrectly installed"
          echo "*** or that you have moved libSigC++ since it was installed. In the latter case, you"
          echo "*** may want to edit the sigc-config script: $LIBSIGC_CONFIG" ])
          CPPFLAGS="$ac_save_CPPFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     LIBSIGC_CPPFLAGS=""
     LIBSIGC_LIBS=""
     AC_SUBST(HAVE_LIBSIGC, 0)
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(LIBSIGC_CPPFLAGS)
  AC_SUBST(LIBSIGC_LIBS)
  AC_LANG_POP([C++])
  rm -f conf.libsigctest
])

  

