
#include <Y/c++.h>
#include <sigc++/bind.h>
#include "calculator.h"

#include <iostream>
#include <sstream>
#include <cmath>

using namespace SigC;
using namespace std;

Calculator::Calculator ()
{
  window = new Y::Window ();
  window -> setProperty ("title", "Calculator");

  Y::GridLayout *grid = new Y::GridLayout ();
  window -> setChild (grid);

  window -> requestClose.connect (bind (slot (&exit), EXIT_SUCCESS));

  result = new Y::Label ();
  result -> setProperty ("text", "0.");
  result -> setProperty ("alignment", "right");
  grid -> addWidget (result, 0, 0, 4, 1);

  Y::Button *button0 = new Y::Button ();
  button0 -> setProperty ("text", "0");
  button0 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 0));
  grid -> addWidget (button0, 0, 5, 2, 1);

  Y::Button *button1 = new Y::Button ();
  button1 -> setProperty ("text", "1");
  button1 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 1));
  grid -> addWidget (button1, 0, 4);

  Y::Button *button2 = new Y::Button ();
  button2 -> setProperty ("text", "2");
  button2 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 2));
  grid -> addWidget (button2, 1, 4);

  Y::Button *button3 = new Y::Button ();
  button3 -> setProperty ("text", "3");
  button3 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 3));
  grid -> addWidget (button3, 2, 4);

  Y::Button *button4 = new Y::Button ();
  button4 -> setProperty ("text", "4");
  button4 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 4));
  grid -> addWidget (button4, 0, 3);

  Y::Button *button5 = new Y::Button ();
  button5 -> setProperty ("text", "5");
  button5 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 5));
  grid -> addWidget (button5, 1, 3);

  Y::Button *button6 = new Y::Button ();
  button6 -> setProperty ("text", "6");
  button6 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 6));
  grid -> addWidget (button6, 2, 3);

  Y::Button *button7 = new Y::Button ();
  button7 -> setProperty ("text", "7");
  button7 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 7));
  grid -> addWidget (button7, 0, 2);

  Y::Button *button8 = new Y::Button ();
  button8 -> setProperty ("text", "8");
  button8 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 8));
  grid -> addWidget (button8, 1, 2);

  Y::Button *button9 = new Y::Button ();
  button9 -> setProperty ("text", "9");
  button9 -> clicked.connect (bind(slot(this, &Calculator::numberClicked), 9));
  grid -> addWidget (button9, 2, 2);

  Y::Button *buttonDot = new Y::Button ();
  buttonDot -> setProperty ("text", ".");
  buttonDot -> clicked.connect (slot(this, &Calculator::pointClicked));
  grid -> addWidget (buttonDot, 2, 5);

  Y::Button *buttonEqu = new Y::Button ();
  buttonEqu -> setProperty ("text", "=");
  buttonEqu -> clicked.connect (slot(this, &Calculator::equalsClicked));
  grid -> addWidget (buttonEqu, 3, 4, 1, 2);

  Y::Button *buttonAdd = new Y::Button ();
  buttonAdd -> setProperty ("text", "+");
  buttonAdd -> clicked.connect
       (bind(slot(this, &Calculator::operatorClicked), PLUS));
  grid -> addWidget (buttonAdd, 3, 2, 1, 2);

  Y::Button *buttonSub = new Y::Button ();
  buttonSub -> setProperty ("text", "-");
  buttonSub -> clicked.connect
       (bind(slot(this, &Calculator::operatorClicked), MINUS));
  grid -> addWidget (buttonSub, 3, 1);

  Y::Button *buttonMul = new Y::Button ();
  buttonMul -> setProperty ("text", "x");
  buttonMul -> clicked.connect
       (bind(slot(this, &Calculator::operatorClicked), MULTIPLY));
  grid -> addWidget (buttonMul, 2, 1);

  Y::Button *buttonDiv = new Y::Button ();
  buttonDiv -> setProperty ("text", "/");
  buttonDiv -> clicked.connect
       (bind(slot(this, &Calculator::operatorClicked), DIVIDE));
  grid -> addWidget (buttonDiv, 1, 1);

  Y::Button *buttonClr = new Y::Button ();
  buttonClr -> setProperty ("text", "C");
  buttonClr -> clicked.connect (slot (this, &Calculator::clearClicked));
  grid -> addWidget (buttonClr, 0, 1);

  current = 0;
  decimalPlace = 0;

  window -> show ();
}

Calculator::~Calculator ()
{
}

void
Calculator::setResult (double v)
{
  double integral;
  ostringstream s;
  s.precision (16);
  s << v;
  if (modf (v, &integral) == 0)
    s << '.';
  result -> setProperty ("text", s.str ());
}

void
Calculator::numberClicked (int n)
{
  if (decimalPlace > 0)
    {
      current += n / (pow (10.0, decimalPlace));
      ++decimalPlace;
    }
  else
    {
      current = (current * 10) + n;
    }
  setResult (current);
}

void
Calculator::pointClicked ()
{
  if (decimalPlace == 0)
    decimalPlace = 1;
  setResult (current);
}

void
Calculator::clearClicked ()
{
  current = 0;
  decimalPlace = 0;
  setResult (current);
}

bool
Calculator::operatorLessThan (Operator op1, Operator op2)
{
  /* { x, / }  <  { +, - }  <  { = }
   */
  if (op1 == MULTIPLY || op1 == DIVIDE)
    {
      if (op2 == MULTIPLY || op2 == DIVIDE)
        return false;
      else
        return true;
    }
  else if (op1 == PLUS || op1 == MINUS)
    {
      if (op2 == EQUALS)
        return true;
      else
        return false;
    }
  else
    return false;
}

void
Calculator::evaluate (Operator max)
{
  while (!operators.empty() && operatorLessThan(operators.top(), max))
    {
       double operand1 = operands.top ();
       operands.pop ();
       double operand2 = operands.top ();
       operands.pop ();
       switch (operators.top ())
         {
           case PLUS:
             operands.push (operand2 + operand1);
             break;
           case MINUS:
             operands.push (operand2 - operand1);
             break;
           case MULTIPLY:
             operands.push (operand2 * operand1);
             break;
           case DIVIDE:
             operands.push (operand2 / operand1);
             break;
         }
       operators.pop ();
    }
}

void
Calculator::operatorClicked (Operator op)
{
  operands.push (current);
  evaluate (op);
  setResult (operands.top ());
  operators.push (op);

  current = 0;
  decimalPlace = 0;
}

void
Calculator::equalsClicked ()
{
  operands.push (current);
  evaluate (EQUALS);
  setResult (operands.top ());
  operands.pop ();

  current = 0;
  decimalPlace = 0;
}

int
main (int argc, char **argv)
{
  Y::Y::initialise (&argc, &argv);
  Calculator c;
  Y::Y::run ();
}

/* arch-tag: 4f1e6992-37c8-438d-b8bf-611aa3bc1ce3
 */
