
#include "clock.h"

#include <iostream>

#include <time.h>
#include <sys/time.h>

#include <sigc++/func_slot.h>
#include <sigc++/bind.h>

#define COS30    (0.8660254)
#define SIN30    (0.5)

using std::cout;
using std::endl;
using SigC::slot;
using SigC::bind;

Clock::Clock ()
{
  width = 64;
  height = 64;

  window = new Y::Window ();
  window -> setProperty ("title", "Clock");

  canvas = new Y::Canvas ();
  canvas -> requestSize (128, 128);

  window -> setChild (canvas);

  window -> requestClose.connect (bind (slot (&exit), EXIT_SUCCESS));

  drawClock ();

  canvas -> resize.connect (slot (this, &Clock::drawClock));

  // canvas -> setProperty ("background", 0xFFC0C0C0);

  Y::Y::timer.connect (slot (this, &Clock::timer));

  Y::Y::time (1000);

  window -> show ();
}

Clock::~Clock ()
{

}

void
Clock::timer ()
{
  drawClock ();
  Y::Y::time (1000);  
}

void
Clock::drawClock ()
{
  canvas -> reset (width, height);

  int r = (width < height) ? width/2 : height/2;
  int cx = width / 2;
  int cy = height / 2;
  int t = r / 8;
  if (t < 2)
    t = 2;
  int s = r - t;
  int ssin30 = (int)(s * SIN30);
  int scos30 = (int)(s * COS30);
  int tsin30 = (int)(t * SIN30);
  int tcos30 = (int)(t * COS30); 

  /* clock face */
  canvas -> setPenColour (0xFF000000);
  canvas -> drawLines (17,
  /* 12 posn */        cx + 1, cy - s, 0, -t,
                       cx + 1, cy - r, t/2, 0,
                       cx + t/2 + 1, cy - r, -t/2, t,
                       cx - 1, cy - s, 0, -t,
                       cx - 1, cy - r, -t/2, 0,
                       cx - t/2 - 1, cy - r, t/2, t,
  /* 1 posn */         cx + ssin30, cy - scos30, tsin30, -tcos30,
  /* 2 posn */         cx + scos30, cy - ssin30, tcos30, -tsin30,
  /* 3 posn */         cx + s, cy, t, 0,
  /* 4 posn */         cx + scos30, cy + ssin30, tcos30, tsin30,
  /* 5 posn */         cx + ssin30, cy + scos30, tsin30, tcos30,
  /* 6 posn */         cx, cy + s, 0, t,
  /* 7 posn */         cx - ssin30, cy + scos30, -tsin30, tcos30,
  /* 8 posn */         cx - scos30, cy + ssin30, -tcos30, tsin30,
  /* 9 posn */         cx - s, cy, -t, 0,
  /* 10 posn */        cx - scos30, cy - ssin30, -tcos30, -tsin30,
  /* 11 posn */        cx - ssin30, cy - scos30, -tsin30, -tcos30
                      );

  time_t now = std::time (NULL);
  struct std::tm *currentTime = std::localtime (&now);
  
  double scAng = currentTime -> tm_sec * M_PI / 30.0;
  double mnAng = currentTime -> tm_min * M_PI / 30.0 + scAng / 60.0;
  double hrAng = currentTime -> tm_hour * M_PI / 6.0 + mnAng / 12.0;

  canvas -> setPenColour (0xFF0000FF);
  canvas -> drawLines (2,
                       cx, cy, (int) (r * 0.5 * sin (hrAng)),
                               (int) (-r * 0.5 * cos (hrAng)),
                       cx, cy, (int) (r * 0.85 * sin (mnAng)),
                               (int) (-r * 0.85 * cos (mnAng)));
  canvas -> setPenColour (0xCCFF0000);
  canvas -> drawLine (cx, cy, (int) (r * 0.95 * sin (scAng)),
                              (int) (-r * 0.95 * cos (scAng)));

  canvas -> swapBuffers ();
}

int
main (int argc, char **argv)
{
  Y::Y::initialise (&argc, &argv);
  Clock *c = new Clock ();
  Y::Y::run ();
}

/* arch-tag: 68a9c23d-bb8b-40d4-8547-b920642a1b4e
 */
