
#include <Y/c++/console.h>
#include <Y/c++/y.h>

#include <sstream>

using std::ostringstream;

int Y::Console::consoleClassId = 0;

Y::Console::Console ()
{
  if (consoleClassId == 0)
    consoleClassId = Y::Y::findClassId ("Console");

  create (consoleClassId);

  subscribeSignal ("keyPress");
  subscribeSignal ("resize");
}

Y::Console::~Console ()
{
}

void
Y::Console::drawText (int col, int row, const string &mbstring,
                      int length, int width)
{
  ostringstream ss;
  ss << "drawText" << Y::sep << col << Y::sep << row
                   << Y::sep << mbstring << Y::sep << length
                   << Y::sep << width;
  invokeMethod (ss.str (), false);
}

void
Y::Console::clearRect (int sCol, int sRow, int eCol, int eRow)
{
  ostringstream ss;
  ss << "clearRect" << Y::sep << sCol << Y::sep << sRow
                    << Y::sep << eCol << Y::sep << eRow;
  invokeMethod (ss.str (), false);
}

void
Y::Console::setRendition (int bold, int blink, int inverse, int underline,
                          int foreground, int background, char charset)
{
  char charsetstring[] = { charset, '\0' };
  ostringstream ss;
  ss << "setRendition" << Y::sep << bold << Y::sep << blink
                       << Y::sep << inverse << Y::sep << underline
                       << Y::sep << foreground << Y::sep << background
                       << Y::sep << charsetstring;
  invokeMethod (ss.str (), false);
}

void
Y::Console::swapVideo ()
{
  invokeMethod ("swapVideo", false);
}

void
Y::Console::ring ()
{
  invokeMethod ("ring", false);
}

void
Y::Console::updateCursorPos (int col, int row)
{
  ostringstream ss;
  ss << "updateCursorPos" << Y::sep << col << Y::sep << row;
  invokeMethod (ss.str (), false);
}

void
Y::Console::scrollView (int destRow, int srcRow, int numLines)
{
  ostringstream ss;
  ss << "scrollView" << Y::sep << destRow << Y::sep << srcRow
                     << Y::sep << numLines;
  invokeMethod (ss.str (), false);
}

void
Y::Console::onEvent (const string &name, const vector<string> &params)
{
  if (name == "keyPress" && params.size () == 2)
    {
      enum YKeyCode keyId =
        (enum YKeyCode) strtoul (params[0].c_str (), NULL, 0);
      unsigned int modState = strtoul (params[1].c_str (), NULL, 0);
      keyPress (keyId, modState);
    }
  else if (name == "resize" && params.size () == 2)
    {
      int cols = strtol (params[0].c_str (), NULL, 0);
      int rows = strtol (params[1].c_str (), NULL, 0);
      resize (cols, rows);
    }
}

/* arch-tag: 3c4bea9a-db16-4a53-b40c-317e2ab8a93a
 */
