
#include <Y/c++/y.h>
#include <Y/c++/object.h>

#include <Y/libY.h>

#include <iostream>
#include <cstring>

#include <stdio.h>
#include <stdlib.h>

using std::cout;
using std::endl;

Y::Y *              Y::Y::instance = NULL;
SigC::Signal0<void> Y::Y::timer; 

Y::Y::Y (int *argcp, char ***argvp)
{
  yInitialise ();
  timeout = false;
}

Y::Y::~Y ()
{
  yFinalise ();
}

void
Y::Y::initialise (int *argcp, char ***argvp)
{
  if (instance == NULL)
    instance = new Y (argcp, argvp);
}

void
Y::Y::finalise ()
{
  if (instance != NULL)
    delete instance;
}

int
Y::Y::findClassId (const char *className)
{
  struct Message req, rep;

  req.to = 0;
  req.cid = 0;
  req.oid = 0;
  req.op = YMO_FIND_CLASS;
  req.meta = 0;
  req.ext = 0;
  req.data = const_cast<char *>(className);
  req.len = std::strlen (className) + 1;

  sendMessage (&req);
  recvMessage (&rep);

  return rep.cid;

}

void
Y::Y::time (int msec)
{
  struct timeval now;
  gettimeofday (&now, NULL);
  instance -> timeoutAt.tv_usec = (now.tv_usec + msec * 1000) % 1000000;
  instance -> timeoutAt.tv_sec = now.tv_sec +
                                (now.tv_usec + msec * 1000) / 1000000;
  instance -> timeout = true;
}

vector<string>
Y::Y::parseParams (char *data, int len)
{
  char *c = data;
  size_t s;
  vector<string> params;
  if (c != NULL)
    while (*c != '\0' && len > 0)
      {
        s = std::strcspn (c, "\x1c");
        params.push_back (string (c, s));
        len -= s + 1;
        c += s + 1;
      }
  return params;
}

void
Y::Y::processEvent (struct Message *ev)
{
  if (ev -> cid != 0 && ev -> oid != 0)
    {
      Object *object = Object::find (ev -> oid);
      if (object != NULL)
        {
          vector<string> params = parseParams (ev -> data, ev -> len);
          if (!params.empty ())
            {
              string name = params[0];
              params.erase (params. begin ());
              object -> onEvent (name, params);
            }
        }
    }
}

void
Y::Y::run ()
{
  for (;;)
    {
      struct Message m;
      int t = -1;
      int r = 0;
      if (instance -> timeout)
        {
          struct timeval now;
          gettimeofday (&now, NULL);
          t = (instance -> timeoutAt.tv_sec - now.tv_sec) * 1000 +
              (instance -> timeoutAt.tv_usec - now.tv_usec) / 1000;
          if (t < 1)
            {
              instance -> timeout = false;
              timer ();
              continue;
            } 
        }
      if (!instance -> events.empty ())
        {
          struct Message *ev = instance -> events.front ();
          instance -> events.pop_front ();
          processEvent (ev);
          ::free (ev -> data);
          delete ev;
          continue;
        }
      r = yReceive (&m, t);
      if (r)
        {
          switch (m.op)
            {
              case YMO_EVENT:
                processEvent (&m);
                break;
            }
          ::free (m.data);
        }
    }
}

void
Y::Y::sendMessage (struct Message *m)
{
  ySend (m);
}

void
Y::Y::recvMessage (struct Message *m)
{
  for (;;)
    {
       yReceive (m, -1);
       if (m -> op == YMO_NO_OPERATION)
         continue;
       if (m -> op == YMO_EVENT)
         {
           struct Message *ev = new struct Message;
           ::memcpy (ev, m, sizeof (struct Message));
           instance -> events.push_back (ev);
           m -> len = 0;
           m -> data = NULL;
           continue;
         }
       return;
    }
}

/* arch-tag: 39a41201-2e34-48ef-aa33-9c6cc5b2c74b
 */
