
#include <Y/modules/module_interface.h>
#include <Y/modules/windowmanager_interface.h>
#include <Y/object/object.h>
#include <Y/widget/desktop.h>
#include <Y/screen/screen.h>
#include <Y/util/zorder.h>
#include <Y/util/yutil.h>
#include <stdio.h>

static void defaultSelectWindow (struct WindowManager *wm, struct Window *w);

static struct Desktop *desktop;
static struct Window *currentWindow;

static void
defaultUnload (struct WindowManager *wm)
{
  moduleUnload ("wm/default");
}

static void
defaultRegisterWindow (struct WindowManager *wm, struct Window *w)
{
  desktopAddWindow (desktop, w);
}

static void
defaultUnregisterWindow (struct WindowManager *wm, struct Window *w)
{
  if (currentWindow == w)
    currentWindow = NULL;
  desktopRemoveWindow (desktop, w);
}

static void
defaultKeyboard (struct WindowManager *wm, int key, int pressed, int modifiers)
{
  if (pressed)
    {
      switch (key)
        {
          case YK_TAB:
             desktopCycleWindows (desktop, (modifiers & YMOD_SHIFT) ? -1 : 1);
             break;
          case YK_W:
          case YK_w:
          case YK_F4:
             if (currentWindow != NULL)
               windowRequestClose (currentWindow);
             break;
        }
    }
}

static void
defaultWindowPointerButton (struct WindowManager *wm, struct Window *w,
                            int x, int y, int button, int pressed)
{
  if (pressed == 1)
    {
      defaultSelectWindow (wm, w);
    }
}

static void
defaultWindowPointerMotion (struct WindowManager *wm, struct Window *w,
                            int x, int y, int dx, int dy)
{
}

static void
defaultWindowPointerEnter (struct WindowManager *wm, struct Window *w,
                           int x, int y)
{
}

static void
defaultWindowPointerLeave (struct WindowManager *wm, struct Window *w)
{
}

static struct Window *
defaultSelectedWindow (struct WindowManager *wm)
{
  return currentWindow;
}

static void
defaultSelectWindow (struct WindowManager *wm, struct Window *w)
{
  if (currentWindow != w)
    {
      if (currentWindow != NULL)
        {
          widgetRepaint (windowToWidget (currentWindow), NULL);
          keyboardRemoveFocus (windowGetFocussedWidget (currentWindow));
        }
      currentWindow = w;
      if (currentWindow != NULL)
        {
          keyboardSetFocus (windowGetFocussedWidget (currentWindow));
          widgetRepaint (windowToWidget (currentWindow), NULL);
          desktopRaiseWindow (desktop, currentWindow);
        }
    }
}

static void
defaultMaximiseWindow (struct WindowManager *wm, struct Window *win)
{
  int x, y, w, h;
  defaultSelectWindow (wm, win);
  widgetGetPosition (desktopToWidget (desktop), &x, &y);
  widgetGetSize (desktopToWidget (desktop), &w, &h);
  windowSaveGeometry (win);
  windowSetSizeState (win, WINDOW_SIZE_MAXIMISE);
  widgetMove (windowToWidget (win), x, y);
  widgetResize (windowToWidget (win), w, h);
}

static void
defaultRestoreWindow (struct WindowManager *wm, struct Window *win)
{
  defaultSelectWindow (wm, win);
  windowSetSizeState (win, WINDOW_SIZE_NORMAL);
  windowRestoreGeometry (win);
}

int
initialise (struct Module *m, int argc, const char **argv)
{
  struct WindowManager *self;
  self = ymalloc (sizeof (struct WindowManager));
  self -> unload              = defaultUnload;
  self -> registerWindow      = defaultRegisterWindow;
  self -> unregisterWindow    = defaultUnregisterWindow;
  self -> keyboard            = defaultKeyboard;
  self -> windowPointerButton = defaultWindowPointerButton;
  self -> windowPointerMotion = defaultWindowPointerMotion;
  self -> windowPointerEnter  = defaultWindowPointerEnter;
  self -> windowPointerLeave  = defaultWindowPointerLeave;
  self -> selectedWindow      = defaultSelectedWindow;
  self -> selectWindow        = defaultSelectWindow;
  self -> maximiseWindow      = defaultMaximiseWindow;
  self -> restoreWindow       = defaultRestoreWindow;
  self -> module = m;
  m -> name = "Default Window Manager";
  m -> data = self;

  windowmanagerRegister (self);

  desktop = desktopCreate ();
  screenSetRootWidget (desktopToWidget (desktop));
  return 0;
}

int
finalise (struct Module *m)
{
  desktopDestroy (desktop);
  yfree (m -> data);
  return 0;
}

/* arch-tag: bea4f81c-1f2d-41ba-be84-362a400a0cb0
 */
